@@ -29,7 +29,7 @@ use self::{hoist_decls::DeclHoisterConfig, optimize::optimizer};
 use crate::{
     analyzer::{analyze, UsageAnalyzer},
     compress::hoist_decls::decl_hoister,
-    debug::dump,
+    debug::{dump, AssertValid},
     marks::Marks,
     mode::Mode,
     option::CompressOptions,
@@ -179,7 +179,10 @@ where
 
     fn optimize_unit_repeatedly<N>(&mut self, n: &mut N)
     where
-        N: CompileUnit + VisitWith<UsageAnalyzer> + for<'aa> VisitMutWith<Compressor<'aa, M>>,
+        N: CompileUnit
+            + VisitWith<UsageAnalyzer>
+            + for<'aa> VisitMutWith<Compressor<'aa, M>>
+            + VisitWith<AssertValid>,
     {
         if cfg!(feature = "debug") {
             tracing::debug!(
@@ -227,7 +230,10 @@ where
     /// Optimize a module. `N` can be [Module] or [FnExpr].
     fn optimize_unit<N>(&mut self, n: &mut N)
     where
-        N: CompileUnit + VisitWith<UsageAnalyzer> + for<'aa> VisitMutWith<Compressor<'aa, M>>,
+        N: CompileUnit
+            + VisitWith<UsageAnalyzer>
+            + for<'aa> VisitMutWith<Compressor<'aa, M>>
+            + VisitWith<AssertValid>,
     {
         let _timer = timer!("optimize", pass = self.pass);
 
@@ -335,6 +341,10 @@ where
             }
         }
 
+        if cfg!(debug_assertions) {
+            n.visit_with(&mut AssertValid);
+        }
+
         {
             let _timer = timer!("apply full optimizer");
 
